@@ -20,7 +20,9 @@
 #define INC_SPF_DNS
 
 
-/*
+/**
+ * @file
+ *
  * This library has the ability to use one or more of any of a number
  * of DNS "layers" to obtain DNS information.  These layers all have
  * compatible APIs, with only minor differences that relate to their
@@ -66,6 +68,7 @@
  * For those who don't have <arpa/nameserv.h>
  */
 
+/* XXX This should use a more sensible define. */
 #if !defined( HAVE_NS_TYPE )
 
 #define	ns_t_invalid	0
@@ -82,7 +85,11 @@
 typedef int	ns_type;
 #endif
 
-#ifndef ns_t_invalid
+#if ! HAVE_DECL_NS_T_SPF
+#define ns_t_spf	99
+#endif
+
+#if ! HAVE_DECL_NS_T_INVALID
 #define	ns_t_invalid	0
 #endif
 
@@ -127,6 +134,11 @@ typedef int (*SPF_dns_add_cache_t)( SPF_server_t *spf_server,
 
 struct SPF_dns_server_struct
 {
+	/** The destructor for this SPF_dns_server_t. If this is NULL, then
+	 * the structure is assumed to be shared between multiple SPF_server_t
+	 * objects, and is not freed when the server is destroyed, or by any call
+	 * to SPF_dns_free(). In this case, it is assumed that somebody else knows,
+	 * and will free the resolver at the appropriate object. */
     SPF_dns_destroy_t	 destroy;
 
     SPF_dns_lookup_t	 lookup;
@@ -155,7 +167,8 @@ SPF_dns_rr_t	*SPF_dns_rlookup6( SPF_dns_server_t *spf_dns_server,
 				int should_cache );
 
 
-/* The client domain is the validated domain name of the client IP
+/**
+ * The client domain is the validated domain name of the client IP
  * address.  This is not just the domain name(s) found in the reverse
  * DNS tree, but involves checking to make sure these name(s) use the
  * client IP address.  The complete validation procedure is described
@@ -164,18 +177,5 @@ SPF_dns_rr_t	*SPF_dns_rlookup6( SPF_dns_server_t *spf_dns_server,
 char		*SPF_dns_get_client_dom(SPF_dns_server_t *spf_dns_server,
 				SPF_request_t *sr);
 
-/*
- * helper functions
- */
-
-
-#if 0
-#ifdef WORDS_BIGENDIAN
-#define SPF_IPV4(a,b,c,d) ((unsigned long)((((unsigned long)a)<<24) + (((unsigned long)b)<<16) + (((unsigned long)c)<<8) + (unsigned long)d))
-#else
-#define SPF_IPV4(a,b,c,d) ((unsigned long)((((unsigned long)d)<<24) + (((unsigned long)c)<<16) + (((unsigned long)b)<<8) + (unsigned long)a))
-#endif
-#endif
-
 
 #endif
