@@ -14,6 +14,7 @@
  */
 
 #include "spf_sys_config.h"
+#include "spf_internal.h"
 
 
 #ifdef STDC_HEADERS
@@ -53,7 +54,8 @@ enum SPF_domspec_enum {
 	DOMSPEC_NONE, DOMSPEC_OPTIONAL, DOMSPEC_REQUIRED
 } SPF_domspec_t;
 
-/* This is greater than any possible total mechanism or modifier.
+/**
+ * This is greater than any possible total mechanism or modifier.
  *	 SPF_MAX_MOD_LEN  + SPF_MAX_STR_LEN
  *	 SPF_MAX_MECH_LEN + SPF_MAX_STR_LEN
  */
@@ -67,7 +69,7 @@ struct SPF_mechtype_struct
 	unsigned char		 mech_type;
 	unsigned char		 is_dns_mech;
 	SPF_domspec_t		 has_domainspec;
-	SPF_cidr_t				 has_cidr;
+	SPF_cidr_t			 has_cidr;
 } SPF_mechtype_t;
 
 static const SPF_mechtype_t spf_mechtypes[] = {
@@ -89,29 +91,36 @@ static const SPF_mechtype_t spf_mechtypes[] = {
 static const SPF_mechtype_t *
 SPF_mechtype_find(int mech_type)
 {
-	int		 i;
+	size_t		 i;
 	for (i = 0; i < spf_num_mechanisms; i++) {
-			if (spf_mechtypes[i].mech_type == mech_type)
+		if (spf_mechtypes[i].mech_type == mech_type)
 			return &spf_mechtypes[i];
 	}
 	return NULL;
 }
 
-static void
-SPF_c_ensure_capacity(void **datap, int *sizep, int length)
+WARN_UNUSED_RESULT
+static int
+SPF_c_ensure_capacity(void **datap, size_t *sizep, size_t length)
 {
-	int		 size = *sizep;
+	size_t		 size = *sizep;
 	if (length > size)
 		size = length + (length / 4);
 	if (size > *sizep) {
-		*datap = realloc(*datap, size);
+		void	*tmp = realloc(*datap, size);
+		if (!tmp)
+			return -1;
+		*datap = tmp;
 		*sizep = size;
 	}
+	return 0;
 }
 
-/* If a struct for IP addresses is added which itself contains a
+/**
+ * If a struct for IP addresses is added which itself contains a
  * CIDR field, then this must be modified to take a (cidr *) rather
- * than a (SPF_data_cidr_t *) */
+ * than a (SPF_data_cidr_t *)
+ */
 static SPF_errcode_t
 SPF_c_parse_cidr_ip6(SPF_response_t *spf_response,
 				unsigned char *maskp,
@@ -435,7 +444,7 @@ SPF_c_parse_var(SPF_response_t *spf_response, SPF_data_var_t *data,
 static SPF_errcode_t
 SPF_c_parse_macro(SPF_server_t *spf_server,
 				SPF_response_t *spf_response,
-				SPF_data_t *data, int *data_len,
+				SPF_data_t *data, size_t *data_len,
 				const char **startp, const char **endp,
 				size_t max_len, SPF_errcode_t big_err,
 				int is_mod)
@@ -448,7 +457,7 @@ SPF_c_parse_macro(SPF_server_t *spf_server,
 	size_t				len;
 			/* For parsing strings. */
 	char				*dst;
-	int					 ds_len;
+	size_t				 ds_len;
 
 	start = *startp;
 	end = *endp;
@@ -468,10 +477,12 @@ SPF_c_parse_macro(SPF_server_t *spf_server,
 		if (len > 0) {				/* An optimisation */
 			if ( p + len > end )	/* Don't re-parse the CIDR mask */
 				len = end - p;
+			/*
 			if (spf_server->debug)
-				SPF_debugf("Adding string literal (%d): '%*.*s'",
-								len, len, len, p);
-			memcpy( dst, p, len );
+				SPF_debugf("Adding string literal (%lu): '%*.*s'",
+								(unsigned long)len, (int)len, (int)len, p);
+			*/
+			memcpy(dst, p, len);
 			ds_len += len;
 			dst += len;
 			p += len;
@@ -551,10 +562,10 @@ SPF_c_parse_macro(SPF_server_t *spf_server,
 static SPF_errcode_t
 SPF_c_parse_domainspec(SPF_server_t *spf_server,
 				SPF_response_t *spf_response,
-				SPF_data_t *data, int *data_len,
+				SPF_data_t *data, size_t *data_len,
 				const char **startp, const char **endp,
 				size_t max_len, SPF_errcode_t big_err,
-				int cidr_ok, int is_mod)
+				SPF_cidr_t cidr_ok, int is_mod)
 {
 	SPF_errcode_t		 err;
 			/* Generic parsing iterators and boundaries */
@@ -696,6 +707,7 @@ SPF_c_parse_ip6(SPF_response_t *spf_response, SPF_mech_t *mech, char const **sta
 }
 
 
+WARN_UNUSED_RESULT
 static SPF_errcode_t
 SPF_c_mech_add(SPF_server_t *spf_server,
 				SPF_record_t *spf_record, SPF_response_t *spf_response,
@@ -828,9 +840,13 @@ SPF_c_mech_add(SPF_server_t *spf_server,
 	if (err == SPF_E_SUCCESS) {
 		if (mechtype->is_dns_mech)
 			spf_record->num_dns_mech++;
-		SPF_c_ensure_capacity((void **)&spf_record->mech_first,
+		if (SPF_c_ensure_capacity((void **)&spf_record->mech_first,
 							&spf_record->mech_size,
-							spf_record->mech_len + len);
+							spf_record->mech_len + len) < 0)
+			return SPF_response_add_error_ptr(spf_response,
+							SPF_E_NO_MEMORY,
+							NULL, NULL,
+							"Failed to allocate memory for mechanism");
 		memcpy( (char *)spf_record->mech_first + spf_record->mech_len,
 			spf_mechanism,
 			len);
@@ -843,6 +859,7 @@ SPF_c_mech_add(SPF_server_t *spf_server,
 	return err;
 }
 
+WARN_UNUSED_RESULT
 static SPF_errcode_t
 SPF_c_mod_add(SPF_server_t *spf_server,
 				SPF_record_t *spf_record, SPF_response_t *spf_response,
@@ -858,6 +875,10 @@ SPF_c_mod_add(SPF_server_t *spf_server,
 
 	SPF_errcode_t		 err;
 
+	if (spf_server->debug)
+		SPF_debugf("Adding modifier name=%lu@%s, value=%s",
+						(unsigned long)name_len, mod_name, *mod_value);
+
 	memset(buf, 'A', sizeof(buf));
 	memset(spf_modifier, 0, sizeof(SPF_mod_t));
 
@@ -890,9 +911,13 @@ SPF_c_mod_add(SPF_server_t *spf_server,
 
 	/* Copy the thing in. */
 	if (err == SPF_E_SUCCESS) {
-		SPF_c_ensure_capacity((void **)&spf_record->mod_first,
+		if (SPF_c_ensure_capacity((void **)&spf_record->mod_first,
 							&spf_record->mod_size,
-							spf_record->mod_len + len);
+							spf_record->mod_len + len) < 0)
+			return SPF_response_add_error_ptr(spf_response,
+							SPF_E_NO_MEMORY,
+							NULL, NULL,
+							"Failed to allocate memory for modifier");
 		memcpy( (char *)spf_record->mod_first + spf_record->mod_len,
 			spf_modifier,
 			len);
@@ -1021,14 +1046,13 @@ SPF_record_lint(SPF_server_t *spf_server,
 
 
 
-/*
+/**
  * The SPF compiler.
  *
  * It converts the SPF record in string format that is easy for people
  * to deal with into a compact binary format that is easy for
  * computers to deal with.
  */
-
 SPF_errcode_t
 SPF_record_compile(SPF_server_t *spf_server,
 								SPF_response_t *spf_response, 
@@ -1073,7 +1097,7 @@ SPF_record_compile(SPF_server_t *spf_server,
 	 */
 	p = record;
 
-	if ( strncmp( p, SPF_VER_STR, sizeof( SPF_VER_STR )-1 ) != 0 )
+	if (strncasecmp(p, SPF_VER_STR, sizeof(SPF_VER_STR) - 1) != 0)
 		return SPF_response_add_error_ptr(spf_response, SPF_E_NOT_SPF,
 						NULL, p,
 						"Could not find a valid SPF record");
@@ -1085,54 +1109,58 @@ SPF_record_compile(SPF_server_t *spf_server,
 						"Could not find a valid SPF record");
 
 	spf_record = SPF_record_new(spf_server, record);
+	if (spf_record == NULL) {
+		*spf_recordp = NULL;
+		return SPF_response_add_error_ptr(spf_response, SPF_E_NO_MEMORY,
+						NULL, p,
+						"Failed to allocate an SPF record");
+	}
 	spf_record->version = 1;
 	*spf_recordp = spf_record;
 
 	/*
 	 * parse the SPF record
 	 */
-	while( *p != '\0' )
-	{
+	while (*p != '\0') {
 		/* TODO WARN: If it's a \n or a \t */
 		/* skip to the next token */
-		while( *p == ' ' )
+		while (*p == ' ')
 			p++;
 
-		if (*p == '\0' )
+		if (*p == '\0')
 			break;
 
 		/* see if we have a valid prefix */
 		prefix = PREFIX_UNKNOWN;
-		switch( *p )
-		{
-		case '+':
-			prefix = PREFIX_PASS;
-			p++;
-			break;
-			
-		case '-':
-			prefix = PREFIX_FAIL;
-			p++;
-			break;
-			
-		case '~':
-			prefix = PREFIX_SOFTFAIL;
-			p++;
-			break;
-			
-		case '?':
-			prefix = PREFIX_NEUTRAL;
-			p++;
-			break;
+		switch (*p) {
+			case '+':
+				prefix = PREFIX_PASS;
+				p++;
+				break;
+				
+			case '-':
+				prefix = PREFIX_FAIL;
+				p++;
+				break;
+				
+			case '~':
+				prefix = PREFIX_SOFTFAIL;
+				p++;
+				break;
+				
+			case '?':
+				prefix = PREFIX_NEUTRAL;
+				p++;
+				break;
 
-		default:
-			while ( ispunct( (unsigned char)( *p ) ) ) {
-				SPF_response_add_error_ptr(spf_response,
-								SPF_E_INVALID_PREFIX, NULL, p,
-								"Invalid prefix '%c'", *p);
-					p++;
-			}
-			break;
+			default:
+				while (ispunct((unsigned char)(*p))) {
+					SPF_response_add_error_ptr(spf_response,
+									SPF_E_INVALID_PREFIX, NULL, p,
+									"Invalid prefix '%c'", *p);
+						p++;
+				}
+				break;
 		}
 
 		name_start = p;
@@ -1260,8 +1288,9 @@ SPF_record_compile(SPF_server_t *spf_server,
 			err = SPF_c_mech_add(spf_server,
 							spf_record, spf_response,
 							mechtype, prefix, &val_start);
-			if ( err )
-				/* Do nothing. Continue for the next error. */ ;
+			if (err == SPF_E_NO_MEMORY)
+				return err;
+			/* XXX Else do nothing. Continue for the next error. */
 			/* We shouldn't have to worry about the child function
 			 * updating the pointer. So we just use our 'well known'
 			 * copy. */
@@ -1300,8 +1329,9 @@ SPF_record_compile(SPF_server_t *spf_server,
 			err = SPF_c_mod_add(spf_server,
 							spf_record, spf_response,
 							name_start, name_len, &val_start);
-			if ( err )
-				/* Do nothing. Continue for the next error. */ ;
+			if (err == SPF_E_NO_MEMORY)
+				return err;
+			/* XXX Else do nothing. Continue for the next error. */
 			p = val_end;
 			break;
 			
@@ -1370,6 +1400,8 @@ SPF_record_compile_macro(SPF_server_t *spf_server,
 	/* XXX TODO: Tidy this up? */
 	size = sizeof(SPF_macro_t) + spf_macro->macro_len;
 	*spf_macrop = (SPF_macro_t *)malloc(size);
+	if (!*spf_macrop)
+		return SPF_E_NO_MEMORY;
 	memcpy(*spf_macrop, buf, size);
 
 	return SPF_E_SUCCESS;
