@@ -54,8 +54,10 @@
 #include "spf_dns_zone.h"
 
 
-/*
- * this is really little more than a proof-of-concept static zone.
+/**
+ * @file
+ *
+ * This is really little more than a proof-of-concept static zone.
  *
  * The static zone shouldn't just be an unsorted list that must be
  * completely searched each time.  Rather something should be done to
@@ -87,42 +89,60 @@ static inline void *SPF_spfhook2voidp( SPF_dns_zone_config_t *spfhook )
 
 
 
+/**
+ * Setting 'exact' to true causes an exact match, including all wildcards,
+ * and is used for adding records.
+ */
 static SPF_dns_rr_t *
-SPF_dns_find_zone(SPF_dns_server_t *spf_dns_server,
-				const char *domain, ns_type rr_type)
+SPF_dns_zone_find(SPF_dns_server_t *spf_dns_server,
+				const char *domain, ns_type rr_type,
+				int exact)
 {
     SPF_dns_zone_config_t	*spfhook;
     int		i;
 
-	spfhook = SPF_voidp2spfhook( spf_dns_server->hook );
+	spfhook = SPF_voidp2spfhook(spf_dns_server->hook);
 
-    if (strncmp(domain, "*.", 2) == 0) {
-		for( i = 0; i < spfhook->num_zone; i++ ) {
-			if ( spfhook->zone[i]->rr_type == rr_type
-			  && strcmp( spfhook->zone[i]->domain, domain ) == 0 )
-			return spfhook->zone[i];
+	if (spf_dns_server->debug)
+		SPF_debugf("zone: Searching for RR %s (%d)", domain, rr_type);
+
+	/* If the record we want or are adding starts with '*.' then it must match
+	 * exactly. */
+    if (exact || strncmp(domain, "*.", 2) == 0) {
+		for (i = 0; i < spfhook->num_zone; i++) {
+			if (spfhook->zone[i]->rr_type == rr_type
+					&& strcasecmp(spfhook->zone[i]->domain, domain) == 0)
+				return spfhook->zone[i];
 		}
+		if (spf_dns_server->debug)
+			SPF_debugf("zone: Exact not found");
     }
 	else {
-		size_t	domain_len = strlen( domain );
+		/* We are looking up a record, so lookup-matching semantics apply. */
+		size_t	domain_len = strlen(domain);
 
-		for( i = 0; i < spfhook->num_zone; i++ ) {
-			if ( spfhook->zone[i]->rr_type != rr_type
-			  && spfhook->zone[i]->rr_type != ns_t_any )
-			continue;
+		for (i = 0; i < spfhook->num_zone; i++) {
+			if (spfhook->zone[i]->rr_type != rr_type
+					&& spfhook->zone[i]->rr_type != ns_t_any) {
+				if (spf_dns_server->debug)
+					SPF_debugf("zone: Ignoring record rrtype %d",
+							spfhook->zone[i]->rr_type);
+				continue;
+			}
 
-			if ( strncmp( spfhook->zone[i]->domain, "*.", 2 ) == 0 ) {
-				size_t	zdomain_len;
-				zdomain_len = strlen( spfhook->zone[i]->domain ) - 2;
+			if (strncmp(spfhook->zone[i]->domain, "*.", 2) == 0) {
+				size_t	zdomain_len = strlen(spfhook->zone[i]->domain) - 2;
 				if ((zdomain_len <= domain_len)
-					 && strcmp(spfhook->zone[i]->domain + 2,
-						domain + (domain_len - zdomain_len)) == 0 )
+					 && strcasecmp(spfhook->zone[i]->domain + 2,
+						domain + (domain_len - zdomain_len)) == 0)
 					return spfhook->zone[i];
 			}
-			else if (strcmp(spfhook->zone[i]->domain, domain) == 0) {
+			else if (strcasecmp(spfhook->zone[i]->domain, domain) == 0) {
 				return spfhook->zone[i];
 			}
 		}
+		if (spf_dns_server->debug)
+			SPF_debugf("zone: Non-exact not found");
 	}
 
     return NULL;
@@ -137,18 +157,18 @@ SPF_dns_zone_lookup(SPF_dns_server_t *spf_dns_server,
     SPF_dns_zone_config_t	*spfhook;
     SPF_dns_rr_t			*spfrr;
 
-    spfrr = SPF_dns_find_zone( spf_dns_server, domain, rr_type );
-    if (spfrr) {
+	spfrr = SPF_dns_zone_find(spf_dns_server, domain, rr_type, FALSE);
+	if (spfrr) {
 		SPF_dns_rr_dup(&spfrr, spfrr);
 		return spfrr;
 	}
 
-    if (spf_dns_server->layer_below) {
+	if (spf_dns_server->layer_below) {
 		return SPF_dns_lookup(spf_dns_server->layer_below,
 						domain, rr_type, should_cache);
 	}
 
-	spfhook = SPF_voidp2spfhook( spf_dns_server->hook );
+	spfhook = SPF_voidp2spfhook(spf_dns_server->hook);
 	SPF_dns_rr_dup(&spfrr, spfhook->nxdomain);
 
 	return spfrr;
@@ -156,34 +176,36 @@ SPF_dns_zone_lookup(SPF_dns_server_t *spf_dns_server,
 
 
 SPF_errcode_t
-SPF_dns_zone_add_str( SPF_dns_server_t *spf_dns_server,
+SPF_dns_zone_add_str(SPF_dns_server_t *spf_dns_server,
 				const char *domain, ns_type rr_type,
-				SPF_dns_stat_t herrno, const char *data )
+				SPF_dns_stat_t herrno, const char *data)
 {
     SPF_dns_zone_config_t	*spfhook;
-    SPF_dns_rr_t		*spfrr;
+    SPF_dns_rr_t			*spfrr;
 
     int		err;
     int		cnt;
 
-	spfhook = SPF_voidp2spfhook( spf_dns_server->hook );
+	if (rr_type == ns_t_any) {
+		if (data)
+			SPF_error("RR type ANY can not have data.");
+		if (herrno == NETDB_SUCCESS)
+			SPF_error("RR type ANY must return a DNS error code.");
+	}
+
+	spfhook = SPF_voidp2spfhook(spf_dns_server->hook);
 
     /* try to find an existing record */
-    spfrr = SPF_dns_find_zone( spf_dns_server, domain, rr_type );
+    spfrr = SPF_dns_zone_find(spf_dns_server, domain, rr_type, TRUE);
 
     /* create a new record */
 	if ( spfrr == NULL ) {
-		spfrr = SPF_dns_rr_new_init( spf_dns_server,
-						domain, rr_type, 24*60*60, herrno );
-		if ( spfrr == NULL )
-			return SPF_E_NO_MEMORY;
-
+		/* First make sure we have space for it. */
 		if ( spfhook->num_zone == spfhook->zone_buf_len ) {
 			int				new_len;
 			SPF_dns_rr_t	**new_zone;
 			int				i;
 
-
 			new_len = spfhook->zone_buf_len
 					+ (spfhook->zone_buf_len >> 2) + 4;
 			new_zone = realloc( spfhook->zone,
@@ -198,24 +220,17 @@ SPF_dns_zone_add_str( SPF_dns_server_t *spf_dns_server,
 			spfhook->zone = new_zone;
 		}
 
-
+		/* Now make the new record. */
+		spfrr = SPF_dns_rr_new_init(spf_dns_server,
+						domain, rr_type, 24*60*60, herrno);
+		if (spfrr == NULL)
+			return SPF_E_NO_MEMORY;
 		spfhook->zone[spfhook->num_zone] = spfrr;
 		spfhook->num_zone++;
 
-		/* random lexical scope */
-		/* Should this really be in this condition? */
-		{
-			if ( rr_type == ns_t_any ) {
-				if ( data )
-					SPF_error( "RR type ANY can not have data.");
-				if ( herrno == NETDB_SUCCESS )
-					SPF_error( "RR type ANY must return a DNS error code.");
-			}
-
-			/* We succeeded with the add, but with no data. */
-			if ( herrno != NETDB_SUCCESS )
-				return SPF_E_SUCCESS;
-		}
+		/* We succeeded with the add, but with no data. */
+		if (herrno != NETDB_SUCCESS)
+			return SPF_E_SUCCESS;
 	}
 
 #define SPF_RR_TRY_REALLOC(rr, i, s) do { \
@@ -228,47 +243,48 @@ SPF_dns_zone_add_str( SPF_dns_server_t *spf_dns_server,
      */
     cnt = spfrr->num_rr;
 
-	switch( rr_type ) {
-	case ns_t_a:
-		SPF_RR_TRY_REALLOC(spfrr, cnt, sizeof( spfrr->rr[cnt]->a ));
-		err = inet_pton( AF_INET, data, &spfrr->rr[cnt]->a );
-		if ( err <= 0 )
-			return SPF_E_INVALID_IP4;
-		break;
+	switch (rr_type) {
+		case ns_t_a:
+			SPF_RR_TRY_REALLOC(spfrr, cnt, sizeof( spfrr->rr[cnt]->a ));
+			err = inet_pton( AF_INET, data, &spfrr->rr[cnt]->a );
+			if ( err <= 0 )
+				return SPF_E_INVALID_IP4;
+			break;
 
-	case ns_t_aaaa:
-		SPF_RR_TRY_REALLOC(spfrr, cnt, sizeof( spfrr->rr[cnt]->aaaa ));
-		err = inet_pton( AF_INET6, data, &spfrr->rr[cnt]->aaaa );
-		if ( err <= 0 )
-			return SPF_E_INVALID_IP6;
-		break;
+		case ns_t_aaaa:
+			SPF_RR_TRY_REALLOC(spfrr, cnt, sizeof( spfrr->rr[cnt]->aaaa ));
+			err = inet_pton( AF_INET6, data, &spfrr->rr[cnt]->aaaa );
+			if ( err <= 0 )
+				return SPF_E_INVALID_IP6;
+			break;
 
-	case ns_t_mx:
-		SPF_RR_TRY_REALLOC(spfrr, cnt, strlen( data ) + 1);
-		strcpy( spfrr->rr[cnt]->mx, data );
-		break;
+		case ns_t_mx:
+			SPF_RR_TRY_REALLOC(spfrr, cnt, strlen( data ) + 1);
+			strcpy( spfrr->rr[cnt]->mx, data );
+			break;
 
-	case ns_t_txt:
-		SPF_RR_TRY_REALLOC(spfrr, cnt, strlen( data ) + 1);
-		strcpy( spfrr->rr[cnt]->txt, data );
-		break;
+		case ns_t_txt:
+		case ns_t_spf:
+			SPF_RR_TRY_REALLOC(spfrr, cnt, strlen( data ) + 1);
+			strcpy( spfrr->rr[cnt]->txt, data );
+			break;
 
-	case ns_t_ptr:
-		SPF_RR_TRY_REALLOC(spfrr, cnt, strlen( data ) + 1);
-		strcpy( spfrr->rr[cnt]->ptr, data );
-		break;
+		case ns_t_ptr:
+			SPF_RR_TRY_REALLOC(spfrr, cnt, strlen( data ) + 1);
+			strcpy( spfrr->rr[cnt]->ptr, data );
+			break;
 
-	case ns_t_any:
-		if ( data )
-			SPF_error( "RR type ANY can not have data.");
-		if ( herrno == NETDB_SUCCESS )
-			SPF_error( "RR type ANY must return a DNS error code.");
-		SPF_error( "RR type ANY can not have multiple RR.");
-		break;
+		case ns_t_any:
+			if ( data )
+				SPF_error( "RR type ANY can not have data.");
+			if ( herrno == NETDB_SUCCESS )
+				SPF_error( "RR type ANY must return a DNS error code.");
+			SPF_error( "RR type ANY can not have multiple RR.");
+			break;
 
-	default:
-		SPF_error( "Invalid RR type" );
-		break;
+		default:
+			SPF_error( "Invalid RR type" );
+			break;
 	}
 
     spfrr->num_rr = cnt + 1;
@@ -279,17 +295,17 @@ SPF_dns_zone_add_str( SPF_dns_server_t *spf_dns_server,
 
 
 static void
-SPF_dns_zone_free( SPF_dns_server_t *spf_dns_server )
+SPF_dns_zone_free(SPF_dns_server_t *spf_dns_server)
 {
     SPF_dns_zone_config_t	*spfhook;
     int				i;
 
 	SPF_ASSERT_NOTNULL(spf_dns_server);
-	spfhook = SPF_voidp2spfhook( spf_dns_server->hook );
+	spfhook = SPF_voidp2spfhook(spf_dns_server->hook);
 
 	if (spfhook) {
 		if (spfhook->zone) {
-			for( i = 0; i < spfhook->zone_buf_len; i++ ) {
+			for (i = 0; i < spfhook->zone_buf_len; i++) {
 				if (spfhook->zone[i])
 					SPF_dns_rr_free(spfhook->zone[i]);
 			}
@@ -311,18 +327,19 @@ SPF_dns_zone_new(SPF_dns_server_t *layer_below,
     SPF_dns_zone_config_t	*spfhook;
 
     spf_dns_server = malloc(sizeof(SPF_dns_server_t));
-    if ( spf_dns_server == NULL )
+    if (spf_dns_server == NULL)
 		return NULL;
 	memset(spf_dns_server, 0, sizeof(SPF_dns_server_t));
 
-    spf_dns_server->hook = calloc(1, sizeof(SPF_dns_zone_config_t));
-    if ( spf_dns_server->hook == NULL ) {
-		free( spf_dns_server );
+    spf_dns_server->hook = malloc(sizeof(SPF_dns_zone_config_t));
+    if (spf_dns_server->hook == NULL) {
+		free(spf_dns_server);
 		return NULL;
     }
+	memset(spf_dns_server->hook, 0, sizeof(SPF_dns_zone_config_t));
 
     if (name ==  NULL)
-		name = "resolv";
+		name = "zone";
 
     spf_dns_server->destroy      = SPF_dns_zone_free;
     spf_dns_server->lookup       = SPF_dns_zone_lookup;
@@ -337,16 +354,23 @@ SPF_dns_zone_new(SPF_dns_server_t *layer_below,
 
     spfhook->zone_buf_len = 32;
     spfhook->num_zone = 0;
-    spfhook->zone = calloc( spfhook->zone_buf_len, sizeof( *spfhook->zone ) );
+    spfhook->zone = calloc(spfhook->zone_buf_len, sizeof(*spfhook->zone));
 
-    if ( spfhook->zone == NULL ) {
+    if (spfhook->zone == NULL) {
 		free(spfhook);
 		free(spf_dns_server);
 		return NULL;
     }
 
+	/* XXX This might have to return NO_DATA sometimes. */
     spfhook->nxdomain = SPF_dns_rr_new_init(spf_dns_server,
 					"", ns_t_any, 24 * 60 * 60, HOST_NOT_FOUND);
+	if (spfhook->nxdomain == NULL) {
+		free(spfhook->zone);
+		free(spfhook);
+		free(spf_dns_server);
+		return NULL;
+	}
 
     return spf_dns_server;
 }
